==> tttttest-1.0.3.tar.gz: lib/mrtc_flutter_sample.dart <==
library mrtc_flutter_sample;

import 'package:mrtc_flutter/mrtc_flutter.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';

typedef NoArgsCallback =  Function() ;
typedef PushErrCallback = Function(int errCode, String errStr);
typedef PullArgCallback = Function(String pullUrl) ;
typedef PullErrCallback = Function(String pullUrl, int errCode, String errStr);

class MrtcSample {
  final String pushUrl;
  final MRTCSetting pushSetting;

  MrtcSample({@required this.pushUrl, @required this.pushSetting, }):assert(pushUrl != null),assert(pushSetting!=null);

  MRTCLiveView createLiveView({
    NoArgsCallback pushSucceed,
    PushErrCallback pushFailed,
    PushErrCallback pushOnError,
    NoArgsCallback pushDisconnected,
    PullArgCallback pullSucceed,
    PullErrCallback pullFailed,
    PullErrCallback pullOnError,
    PullArgCallback pullDisconnected,
  }) {
    return new MRTCLiveView(
      url: pushUrl,
      setting: pushSetting,
      pushSucceed: pushSucceed,
      pushFailed: pushFailed,
      pushOnError: pushFailed,
      pushDisconnected: pushDisconnected,
      pullSucceed: pullSucceed,
      pullFailed: pullFailed,
      pullOnError: pullOnError,
      pullDisconnected: pullDisconnected,
    );
  }
}

class MRTCLiveView extends StatefulWidget {
  final MRTCSetting setting;
  final String url;
  final MRTCLiveViewState _liveViewState;
  NoArgsCallback pushSucceed;
  PushErrCallback pushFailed;
  PushErrCallback pushOnError;
  NoArgsCallback pushDisconnected;
  PullArgCallback pullSucceed;
  PullErrCallback pullFailed;
  PullErrCallback pullOnError;
  PullArgCallback pullDisconnected;

  MRTCLiveView({
    @required this.url,
    @required this.setting,
    NoArgsCallback pushSucceed,
    PushErrCallback pushFailed,
    PushErrCallback pushOnError,
    NoArgsCallback pushDisconnected,
    PullArgCallback pullSucceed,
    PullErrCallback pullFailed,
    PullErrCallback pullOnError,
    PullArgCallback pullDisconnected,
  }):_liveViewState = MRTCLiveViewState();

  @override
  State<StatefulWidget> createState() {
    return new MRTCLiveViewState();
  }

  void destory() {
    _liveViewState.destory();
  }
}

class MRTCLiveViewState extends State<MRTCLiveView> {
  bool _showFuncBtns = false;
  Widget _renderer;
  MRTCPusher _pusher;
  final Map<String,MRTCPuller> _pullerMap = new Map();
  final Map<String,Widget> _pullerRendererMap = new Map();
  bool _pushing = false;
  bool _flashLightOn = false;
  bool _mute = false;

  @override
  void initState() {
    SystemChrome.setPreferredOrientations([DeviceOrientation.landscapeLeft,DeviceOrientation.landscapeRight,DeviceOrientation.portraitUp]);
    Mrtc.init();

    setupPusher();

    super.initState();
  }

  @override
  void dispose() {
    destory();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return Container(child: Stack(
      children: <Widget>[
        GestureDetector(
          onTap: (){
            setState(() {
              _showFuncBtns = !_showFuncBtns;
            });
          },
        ),
//        _renderer != null?_renderer:Container(),
        Positioned(
          child:Offstage(
            offstage: _showFuncBtns,
            child: Row(
              mainAxisAlignment: MainAxisAlignment.spaceEvenly,
              children: <Widget>[
                GestureDetector(
                  onTap: _switchCamera,
                  child: Image(image: AssetImage('packages/tttttest/images/SwitchCamera.png'),width: 25,height: 25,),
                ),
                GestureDetector(
                  onTap: _switchFlashLight,
                  child: Image(image: AssetImage(_flashLightOn?'packages/tttttest/images/FlashLightOn.png':'images/FlashLightOff.png'),width: 25,height: 25,),
                ),
                GestureDetector(
                  onTap: _switchMicrophone,
                  child: Image(image: AssetImage(_mute?'packages/tttttest/images/MicOff.png':'images/MicOn.png'),width: 25,height: 25,),
                ),
              ],
            ),
          ),
        ),

        Align(
          alignment: Alignment(1, 0),
          child: GestureDetector(
            onTap: (){
              _pushButtonClicked();
            },
            child: Container(child: Image(image: AssetImage(_pushing?'packages/tttttest/images/PushStop.png':'packages/tttttest/images/Start_s.png'),)),
          ),
        ),
        Align(
          alignment: Alignment(0.9, 0.9),
          child: GestureDetector(
            onTap: () {
              _showAddAlert(context: context);
            },
            child: Image(image: AssetImage('packages/tttttest/images/ClickPlus.png')),
          ),
        ),
//        Align(
//          alignment:Alignment(-0.9, 0.9),
//          child: Row(
//            children: _getPullerRenderers(),
//          ),
//        ),
      ],
    ));
  }

  void setupPusher() async {
    _pusher = await Mrtc.createNativePusher(widget.setting);

    _pusher.setCallback(
        onError: (int errCode, String errStr) {
          widget.pushOnError(errCode,errStr);
        },
        onFailed: (int errCode, String errStr) {
          widget.pushFailed(errCode,errStr);
        },
        onConnect: () {
          widget.pushSucceed();
          setState(() {
            _pushing = true;
          });
        },
        onDisconnect: () {
          widget.pushDisconnected();
          setState(() {
            _pushing = false;
          });
        }
    );

    setState(() {
      _renderer = Mrtc.createNativeView((int id) {
        _pusher.startPreview(id);
      });
    });
  }

  void destory(){
    _pusher.destory();
    for (MRTCPuller puller in _pullerMap.values) {
      puller.destory();
    }
  }

  void _pushButtonClicked() {
    if (!_pushing) {
      _pusher.startPush(widget.url);
    } else {
      _pusher.stopPush();
    }
  }

  void _switchCamera() {
    _pusher.switchCamera();
  }

  void _switchFlashLight() {
    setState(() {
      _flashLightOn = !_flashLightOn;
      _pusher.setFlashLight(_flashLightOn);
    });
  }

  void _switchMicrophone() {
    setState(() {
      _mute = !_mute;
      if(_mute) {
        _pusher.mute();
      } else {
        _pusher.unmute();
      }
    });
  }

  void _showAddAlert({@required BuildContext context}) {
    final TextEditingController controller = TextEditingController(text:'mrtc://test-mrtc.myun.tv/live/');
    showDialog(
      context: context,
      builder: (BuildContext cnt) {
        return AlertDialog(
          title: Text('请输入拉流地址'),
          content: TextField(controller: controller,),
          actions: <Widget>[
            FlatButton(onPressed: (){
              Navigator.pop(cnt);
              _addPuller(controller.text);
            },child: Text('确定',style: TextStyle(color: Colors.blue),),
              color: Colors.grey,
            ),
            FlatButton(onPressed: (){
              Navigator.pop(cnt);
            },child: Text('取消',style: TextStyle(color: Colors.blue),),
              color: Colors.grey,
            )
          ],
        );
      },
    );
  }

  void _addPuller(String url) async {
    if (_pullerMap.containsKey(url)) return;

    MRTCPuller puller = await Mrtc.createNativePuller();

    _pullerMap[url] = puller;

    puller.setCallback(
        onConnect: () {
          widget.pullSucceed(url);
        },
        onDisconnect: () {
          widget.pullDisconnected(url);
          setState(() {
            _pullerMap.remove(url);
            _pullerRendererMap.remove(url);
          });
        },
        onFailed: (int errCode,String errStr) {
          widget.pullFailed(url,errCode,errStr);
          setState(() {
            _pullerMap.remove(url);
            _pullerRendererMap.remove(url);
          });
        },
        onError: (int errCode,String errStr) {
          widget.pullOnError(url,errCode,errStr);
        }
    );

    Widget renderer = Mrtc.createNativeView((int id) {
      puller.startPull(url, id);
    });

    _pullerRendererMap[url] = renderer;

    setState(() {

    });
  }

  String _getKeyForRenderer(Widget renderer){
    String url;
    _pullerRendererMap.forEach((String key,Widget val){
      if(val == renderer){
        url = key;
      }
    });
    return url;
  }

  List<Widget> _getPullerRenderers(){
    List<Widget> list = new List();
    if (_pullerRendererMap.values.length > 0 ){
      List<Widget> rendererList = _pullerRendererMap.values.toList();
      for (Widget renderer in rendererList){
        double w = 90;
        double h = w * 16 / 9;
        list.add(SizedBox(
          width: w,
          height: h,
          child: Stack(
            alignment: AlignmentDirectional.topEnd,
            children: <Widget>[
              renderer,
              Align(
                alignment: Alignment(1, -1),
                child: GestureDetector(
                    onTap: () {
                      String url = _getKeyForRenderer(renderer);
                      if(url != null && _pullerMap.containsKey(url)){
                        setState(() {
                          _pullerRendererMap.remove(url);
                          MRTCPuller puller = _pullerMap[url];
                          puller.destory();
                          _pullerMap.remove(url);
                        });
                      }
                    },
                    child: Image(image: AssetImage('packages/tttttest/images/ClearButton.png'))),
              )
            ],
          ),));
      }
    }
    return list;
  }
}

==> tttttest-1.0.4.tar.gz: lib/mrtc_flutter_sample.dart <==
library mrtc_flutter_sample;

import 'package:mrtc_flutter/mrtc_flutter.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';

typedef NoArgsCallback =  Function() ;
typedef PushErrCallback = Function(int errCode, String errStr);
typedef PullArgCallback = Function(String pullUrl) ;
typedef PullErrCallback = Function(String pullUrl, int errCode, String errStr);

class MrtcSample {
  final String pushUrl;
  final MRTCSetting pushSetting;

  MrtcSample({@required this.pushUrl, @required this.pushSetting, }):assert(pushUrl != null),assert(pushSetting!=null);

  MRTCLiveView createLiveView({
    NoArgsCallback pushSucceed,
    PushErrCallback pushFailed,
    PushErrCallback pushOnError,
    NoArgsCallback pushDisconnected,
    PullArgCallback pullSucceed,
    PullErrCallback pullFailed,
    PullErrCallback pullOnError,
    PullArgCallback pullDisconnected,
  }) {
    return new MRTCLiveView(
      url: pushUrl,
      setting: pushSetting,
      pushSucceed: pushSucceed,
      pushFailed: pushFailed,
      pushOnError: pushFailed,
      pushDisconnected: pushDisconnected,
      pullSucceed: pullSucceed,
      pullFailed: pullFailed,
      pullOnError: pullOnError,
      pullDisconnected: pullDisconnected,
    );
  }
}

class MRTCLiveView extends StatefulWidget {
  final MRTCSetting setting;
  final String url;
  final MRTCLiveViewState _liveViewState;
  NoArgsCallback pushSucceed;
  PushErrCallback pushFailed;
  PushErrCallback pushOnError;
  NoArgsCallback pushDisconnected;
  PullArgCallback pullSucceed;
  PullErrCallback pullFailed;
  PullErrCallback pullOnError;
  PullArgCallback pullDisconnected;

  MRTCLiveView({
    @required this.url,
    @required this.setting,
    NoArgsCallback pushSucceed,
    PushErrCallback pushFailed,
    PushErrCallback pushOnError,
    NoArgsCallback pushDisconnected,
    PullArgCallback pullSucceed,
    PullErrCallback pullFailed,
    PullErrCallback pullOnError,
    PullArgCallback pullDisconnected,
  }):_liveViewState = MRTCLiveViewState();

  @override
  State<StatefulWidget> createState() {
    return new MRTCLiveViewState();
  }

  void destory() {
    _liveViewState.destory();
  }
}

class MRTCLiveViewState extends State<MRTCLiveView> {
  bool _showFuncBtns = false;
  Widget _renderer;
  MRTCPusher _pusher;
  final Map<String,MRTCPuller> _pullerMap = new Map();
  final Map<String,Widget> _pullerRendererMap = new Map();
  bool _pushing = false;
  bool _flashLightOn = false;
  bool _mute = false;

  @override
  void initState() {
    SystemChrome.setPreferredOrientations([DeviceOrientation.landscapeLeft,DeviceOrientation.landscapeRight,DeviceOrientation.portraitUp]);
    Mrtc.init();

    setupPusher();

    super.initState();
  }

  @override
  void dispose() {
    destory();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return Container(child: Stack(
      children: <Widget>[
        GestureDetector(
          onTap: (){
            setState(() {
              _showFuncBtns = !_showFuncBtns;
            });
          },
        ),
//        _renderer != null?_renderer:Container(),
        Positioned(
          child:Offstage(
            offstage: _showFuncBtns,
            child: Row(
              mainAxisAlignment: MainAxisAlignment.spaceEvenly,
              children: <Widget>[
                GestureDetector(
                  onTap: _switchCamera,
                  child: Image(image: AssetImage('packages/tttttest/images/SwitchCamera.png'),width: 25,height: 25,),
                ),
                GestureDetector(
                  onTap: _switchFlashLight,
                  child: Image(image: AssetImage(_flashLightOn?'packages/tttttest/images/FlashLightOn.png':'packages/tttttest/images/FlashLightOff.png')),
                ),
                GestureDetector(
                  onTap: _switchMicrophone,
                  child: Image(image: AssetImage(_mute?'packages/tttttest/images/MicOff.png':'packages/tttttest/images/MicOn.png')),
                ),
              ],
            ),
          ),
        ),

        Align(
          alignment: Alignment(1, 0),
          child: GestureDetector(
            onTap: (){
              _pushButtonClicked();
            },
            child: Container(child: Image(image: AssetImage(_pushing?'packages/tttttest/images/PushStop.png':'packages/tttttest/images/Start_s.png'),)),
          ),
        ),
        Align(
          alignment: Alignment(0.9, 0.9),
          child: GestureDetector(
            onTap: () {
              _showAddAlert(context: context);
            },
            child: Image(image: AssetImage('packages/tttttest/images/ClickPlus.png')),
          ),
        ),
//        Align(
//          alignment:Alignment(-0.9, 0.9),
//          child: Row(
//            children: _getPullerRenderers(),
//          ),
//        ),
      ],
    ));
  }

  void setupPusher() async {
    _pusher = await Mrtc.createNativePusher(widget.setting);

    _pusher.setCallback(
        onError: (int errCode, String errStr) {
          widget.pushOnError(errCode,errStr);
        },
        onFailed: (int errCode, String errStr) {
          widget.pushFailed(errCode,errStr);
        },
        onConnect: () {
          widget.pushSucceed();
          setState(() {
            _pushing = true;
          });
        },
        onDisconnect: () {
          widget.pushDisconnected();
          setState(() {
            _pushing = false;
          });
        }
    );

    setState(() {
      _renderer = Mrtc.createNativeView((int id) {
        _pusher.startPreview(id);
      });
    });
  }

  void destory(){
    _pusher.destory();
    for (MRTCPuller puller in _pullerMap.values) {
      puller.destory();
    }
  }

  void _pushButtonClicked() {
    if (!_pushing) {
      _pusher.startPush(widget.url);
    } else {
      _pusher.stopPush();
    }
  }

  void _switchCamera() {
    _pusher.switchCamera();
  }

  void _switchFlashLight() {
    setState(() {
      _flashLightOn = !_flashLightOn;
      _pusher.setFlashLight(_flashLightOn);
    });
  }

  void _switchMicrophone() {
    setState(() {
      _mute = !_mute;
      if(_mute) {
        _pusher.mute();
      } else {
        _pusher.unmute();
      }
    });
  }

  void _showAddAlert({@required BuildContext context}) {
    final TextEditingController controller = TextEditingController(text:'mrtc://test-mrtc.myun.tv/live/');
    showDialog(
      context: context,
      builder: (BuildContext cnt) {
        return AlertDialog(
          title: Text('请输入拉流地址'),
          content: TextField(controller: controller,),
          actions: <Widget>[
            FlatButton(onPressed: (){
              Navigator.pop(cnt);
              _addPuller(controller.text);
            },child: Text('确定',style: TextStyle(color: Colors.blue),),
              color: Colors.grey,
            ),
            FlatButton(onPressed: (){
              Navigator.pop(cnt);
            },child: Text('取消',style: TextStyle(color: Colors.blue),),
              color: Colors.grey,
            )
          ],
        );
      },
    );
  }

  void _addPuller(String url) async {
    if (_pullerMap.containsKey(url)) return;

    MRTCPuller puller = await Mrtc.createNativePuller();

    _pullerMap[url] = puller;

    puller.setCallback(
        onConnect: () {
          widget.pullSucceed(url);
        },
        onDisconnect: () {
          widget.pullDisconnected(url);
          setState(() {
            _pullerMap.remove(url);
            _pullerRendererMap.remove(url);
          });
        },
        onFailed: (int errCode,String errStr) {
          widget.pullFailed(url,errCode,errStr);
          setState(() {
            _pullerMap.remove(url);
            _pullerRendererMap.remove(url);
          });
        },
        onError: (int errCode,String errStr) {
          widget.pullOnError(url,errCode,errStr);
        }
    );

    Widget renderer = Mrtc.createNativeView((int id) {
      puller.startPull(url, id);
    });

    _pullerRendererMap[url] = renderer;

    setState(() {

    });
  }

  String _getKeyForRenderer(Widget renderer){
    String url;
    _pullerRendererMap.forEach((String key,Widget val){
      if(val == renderer){
        url = key;
      }
    });
    return url;
  }

  List<Widget> _getPullerRenderers(){
    List<Widget> list = new List();
//    if (_pullerRendererMap.values.length > 0 ){
//      List<Widget> rendererList = _pullerRendererMap.values.toList();
//      for (Widget renderer in rendererList){
//        double w = 90;
//        double h = w * 16 / 9;
//        list.add(SizedBox(
//          width: w,
//          height: h,
//          child: Stack(
//            alignment: AlignmentDirectional.topEnd,
//            children: <Widget>[
//              renderer,
//              Align(
//                alignment: Alignment(1, -1),
//                child: GestureDetector(
//                    onTap: () {
//                      String url = _getKeyForRenderer(renderer);
//                      if(url != null && _pullerMap.containsKey(url)){
//                        setState(() {
//                          _pullerRendererMap.remove(url);
//                          MRTCPuller puller = _pullerMap[url];
//                          puller.destory();
//                          _pullerMap.remove(url);
//                        });
//                      }
//                    },
//                    child: Image(image: AssetImage('packages/tttttest/images/ClearButton.png'))),
//              )
//            ],
//          ),));
//      }
//    }
    return list;
  }
}
